"use strict";

b4w.register("racchetta", function(exports, require) {

var m_app    = require("app");
var m_cam    = require("camera");
var m_cfg    = require("config");
var m_cont   = require("container");
var m_ctl    = require("controls");
var m_data   = require("data");
var m_scenes = require("scenes");
var m_time   = require("time");
var m_trans  = require("transform");
var m_util   = require("util");
var m_vec3   = require("vec3");

//var JSON = "../../../deploy/assets/code_snippets/camera_animation/camera_animation.json"
  var JSON = "racchetta.json"

var _cam = null;
var _re = /dot/i;
var _animating = false;
var _curr_spot = null;

var _vec3_tmp1 = new Float32Array(3);
var _vec3_tmp2 = new Float32Array(3);
var _vec3_tmp3 = new Float32Array(3);


exports.init = function() {
    console.log("init");
    m_app.init({
        canvas_container_id:    "canvas_cont",
        callback:               init_cb,
        physics_enabled:        false,
        alpha:                  true,
        show_fps:               true,
        autoresize:             true,
        console_verbose:        true
    });
}

function init_cb(canvas_elem, success) {

    if (!success) {
        console.log("b4w init failure");
        return;
    }
    m_data.load( JSON, load_cb);
}

function load_cb(data_id) {
    m_app.enable_camera_controls();
    _cam = m_scenes.get_active_camera();

    var main_canvas = m_cont.get_canvas();
    main_canvas.addEventListener("mouseup", on_mouse_up);
}


function on_mouse_up(e) {

    if (e.button != 0)
        return;

    if (_animating )
        return;

    if (e.preventDefault)
        e.preventDefault(); // ??

    var obj = m_scenes.pick_object(e.clientX, e.clientY);

    var name = "none";
    if( obj ) name = m_scenes.get_object_name(obj);
    var spot_picked = ( name.match(_re) != null );
    console.log("main_canvas_up", e.clientX, e.clientY, spot_picked, name  );

    if( !spot_picked )
        return;

    _curr_spot = obj;
    _animating = true;

    var pos0   = m_trans.get_translation(_cam, _vec3_tmp1);
    var pos1   = m_trans.get_translation( obj, _vec3_tmp2);
    pos1[1] += 1.6; //pos0[1]; // keep camera distance from ground


    m_time.animate(0, 1, 500, function(e) {

        var pos = m_vec3.lerp(pos0, pos1, m_util.smooth_step(e), _vec3_tmp3);

        m_cam.eye_set_look_at(_cam, pos );

        if (e == 1)
            _animating = false;
    })
}


});

b4w.require("racchetta").init();
